==> crud-el-0.2.29.tgz: package/src/CrudEl/Component/UseCrud.ts <==
import { computed, reactive, ref, toRefs, Ref } from 'vue'
import { ElMessage, ElMessageBox, ElNotification } from 'element-plus'
// 只做初始化引用
import { DefParams, DefCrudState, DefCrudProps } from './DefConfig'
import { DataMerger, DeepClone, isObjEqual } from './Methods'
import { useHooks, hooksName } from './Hooks'

/** 经过main.ts初始化后去除响应式的DefParams值 */
export let defParams: SystemParametersType

/** main初始化时传入的函数 */
export const CrudRequest = ref()
/** 系统参数 */
const Params = ref<SystemParametersType>({ ...DefParams })
/** 组件参数 */
const CrudState = ref<CrusStateType>({ ...DefCrudState })
/** 组件Props */
const CrudProps = ref<CrudPropsType>({ ...DefCrudProps })

/** 系统培训初始化函数 */
export const InitSys = (Options: any) => {
    CrudRequest.value = Options.CrudRequest
    Params.value = DataMerger(DefParams, Options)
    defParams = { ...Params.value }
}
/** Props传参初始化 */
export const Initializer = (props: any) => {
    CrudProps.value = { ...props }
    CrudState.value = DeepClone(DefCrudState)
    // 初始化查询、编辑、表格列
    initElementArrs(CrudProps.value!.source)
    // 重置系统参数
    Params.value = DataMerger(Params.value, props)
}
// 必须使用toRefs，不然参数就没有响应式了，
/** 系统参数 */
export function UseParam() {
    return { ...toRefs(Params.value) }
}
/** 组件参数 */
export function UseState() {
    return { ...toRefs(CrudState.value) }
}
/** Props参数 */
export function UseProps() {
    return { ...toRefs(CrudProps.value) }
}
/** 增删改查等操作方法 */
export const UseFun = () => {
    /**
     * 在JSON中提取出主键对象
     * @param data 提交时的JSON对象，包含主键参数、值
     * @returns 返回包含主键的对象
     */
    const KeyHandling = (data: any) => {
        /** 返回包含主键的对象 */
        let keys: any = {}
        if (CrudProps.value.rowKey) keys[CrudProps.value.rowKey] = data[CrudProps.value.rowKey]
        if (CrudProps.value.rowKeys) {
            CrudProps.value.rowKeys.forEach((key: any) => {
                keys[key] = data[key]
            })
        }
        return keys
    }
    const { hookHandler } = useHooks(CrudProps.value.hooks)

    /** 查询函数 */
    function queryRecord() {
        CrudState.value.tableLoading = true

        /** 查询请求携带的数据 */
        const Data = reactive({ ...CrudState.value.initSearchModel, ...CrudState.value.searchModel })

        for (const key in Data) {
            // 处理下拉选择器清空后，携带空字符串发起请求的问题
            if (typeof Data[key] === 'string') {
                Data[key] = Data[key].trim()
                if (Data[key] === '') {
                    Data[key] = undefined
                }
            }
            if (Data[key] === null) {
                Data[key] = undefined
            }
        }

        // 查询前钩子
        if (!hookHandler(hooksName.beforeQueryHandler, { CrudState: CrudState, Data })) {
            return
        }

        // 开始查询
        CrudRequest.value({
            url: CrudProps.value.url,
            name: Params.value.ApiName.query,
            data: {
                pagenum: Params.value.Page_PageParams.current, //当前页
                pagesize: Params.value.Page_PageParams.pageSize, //分页大小
                ...Data // 查询参数
            }
        })
            .then((Res: { data?: any; page?: any }) => {
                /** 解构出后端返回的 data数组 与 page分页参数 */
                const { data, page } = Res
                // 处理响应的数据（表格赋值等）
                setTimeout(() => {
                    // 为了保证钩子的数据格式统计，加了一层
                    responseData(data, Data)
                    // 更新分页数据
                    Params.value.Page_PageParams.total = page.total
                    // 关闭表格加载
                    CrudState.value.tableLoading = false
                }, 50)
            })
            .catch(() => {
                CrudState.value.tableLoading = false
            })
    }
    /** 查询后 响应数据处理 */
    const responseData = (Data: any, reqDataSource: any) => {
        // 如果请求的是原始数据，就存下来（原始数据判断依据：请求没有添加任何参数）
        let originalArray = Object.keys(reqDataSource).filter((r) => reqDataSource[r] != undefined)
        if (originalArray.length == 0) {
            CrudState.value.originalData = Data
        }
        CrudState.value.originalData = DeepClone(Data)

        // 查询后钩子
        if (!hookHandler(hooksName.afterQueryHandler, { CrudState: CrudState, Data })) {
            return
        }

        // 表格赋值
        CrudState.value.tableData = DeepClone(Data)
    }
    /**点击新增按钮，打开编辑对话框，并初始化值 */
    function addRecord() {
        // 初始化新增时表单的数据
        CrudState.value.editModel = DeepClone(CrudState.value.initEditModel)
        const Data = reactive({ ...CrudState.value.editModel })

        if (!hookHandler(hooksName.beforeAddHandler, { CrudState: CrudState, Data })) {
            return
        }
        CrudState.value.Mode = 'add'
        CrudState.value.DialogVisible = true
    }
    /**点击编辑按钮，打开编辑对话框【把请求到的值赋值给 editModel】 */
    const editRecord = (row: any) => {
        // 获取到当前行的数据
        const Data = reactive({
            ...row
        })

        if (!hookHandler(hooksName.beforeEditHandler, { CrudState: CrudState, Data })) {
            return
        }

        const DeepData = DeepClone(Data)
        // 赋值给编辑表单
        Object.keys(DeepData).forEach((key) => {
            CrudState.value.editModel[key] = DeepData[key]
            CrudState.value.editFormValue[key] = DeepData[key]
        })

        CrudState.value.Mode = 'edit'
        CrudState.value.DialogVisible = true
    }
    /**处理表单提交动作 */
    const handleSubmit = async (formEl: any) => {
        if (!formEl) return
        await formEl.validate((valid: any) => {
            if (valid) {
                const Data = reactive({
                    ...CrudState.value.editModel
                })

                if (!hookHandler(hooksName.beforeSubmitHandler, { CrudState: CrudState, Data })) {
                    return
                }

                // 判断表单提交操作是新增还是编辑
                if (CrudState.value.Mode === 'add') {
                    handleAdd(Data)
                } else {
                    handleEdit(Data)
                }
            }
        })
    }
    /**处理表单新增动作 */
    const handleAdd = (value: any) => {
        CrudState.value.submitLoading = true

        let data = value

        // 判断编辑是否有 FormData 对象
        if (value.FormData instanceof FormData) {
            data = value.FormData
        }

        CrudRequest.value({ url: CrudProps.value.url, name: Params.value.ApiName.add, data })
            .then(() => {
                CrudState.value.DialogVisible = false
                CRUD_Notification('新增成功')
                queryRecord()
                CrudState.value.submitLoading = false
            })
            .catch(() => {
                // 请求数据时，后端返回错误代码
                CrudState.value.submitLoading = false
            })
    }
    /**处理表单编辑动作 */
    const handleEdit = (value: any) => {
        CrudState.value.submitLoading = false

        /** 最终提交时的参数 */
        let data = value

        // 判断编辑是否有 FormData 对象
        if (value.FormData instanceof FormData) {
            data = value.FormData
        } else {
            // 主要解决了编辑表单时，只提交修改项
            const Equal = isObjEqual(CrudState.value.editFormValue, data)
            // 没有编辑项时，就不发送请求
            if (JSON.stringify(Equal) === '{}') {
                CrudState.value.submitLoading = false
                return ElMessage({ message: '没有提交项！', grouping: true })
            }

            data = {
                ...KeyHandling(value),
                ...Equal
            }
        }

        CrudRequest.value({ url: CrudProps.value.url, name: Params.value.ApiName.edit, data })
            .then(() => {
                CrudState.value.DialogVisible = false
                CRUD_Notification('编辑成功')
                queryRecord()
                CrudState.value.submitLoading = false
            })
            .catch(() => {
                // 请求数据时，后端返回错误代码
                CrudState.value.submitLoading = false
            })
    }
    /**标记删除 */
    const delFlag = (row: any) => {
        ElMessageBox.confirm(`确定要 “${row.deleteFlag ? '启用' : '禁用'}” 该条数据吗？`, '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                const Data = reactive({
                    ...KeyHandling(row),
                    deleteFlag: row.deleteFlag == 0 ? 1 : 0
                })
                if (!hookHandler(hooksName.beforedisabledHandler, { CrudState: CrudState, Data })) {
                    return
                }
                CrudRequest.value({
                    url: CrudProps.value.url,
                    name: Params.value.ApiName.deleteFlag,
                    data: Data
                }).then(() => {
                    CRUD_Notification(`数据${row.deleteFlag ? '启用' : '禁用'}成功`)
                    queryRecord()
                })
            })
            .catch(() => {})
    }
    /**删除单行数据 */
    const deleteRecord = (row: any) => {
        ElMessageBox.confirm('确定要删除本条数据吗？', '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                // 获取到需要删除的当前行的数据
                const Data = reactive({
                    ...KeyHandling(row)
                })

                // 删除前钩子
                if (!hookHandler(hooksName.beforeDeteleHandler, { CrudState: CrudState, Data })) {
                    return
                }
                CrudRequest.value({
                    url: CrudProps.value.url,
                    name: Params.value.ApiName.delete,
                    data: Data
                }).then(() => {
                    CRUD_Notification('删除成功')
                    queryRecord()
                })
            })
            .catch(() => {})
    }
    /**删除多行数据 */
    const deleteMulRecord = () => {
        // 整理批量删除的 id
        const multipleSelection = CrudState.value.multipleSelection.map((item: any) => {
            return item[CrudProps.value.rowKey!]
        })

        ElMessageBox.confirm(`确定要删除这 ${multipleSelection.length} 条数据吗？`, '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                // 获取到需要删除的当前行的数据
                const Data = reactive({
                    [CrudProps.value.rowKey!]: multipleSelection,
                    selection: CrudState.value.multipleSelection
                })

                // 批量删除之前钩子
                if (!hookHandler(hooksName.beforeBatchDeteleHandler, { CrudState: CrudState, Data })) {
                    return
                }

                CrudRequest.value({
                    url: CrudProps.value.url,
                    name: Params.value.ApiName.remove,
                    data: Data
                }).then(() => {
                    CRUD_Notification('批量删除成功')
                    CrudState.value.multipleSelection = []
                    //把已经选择的数据清空
                    CrudState.value.CompRef.TableRef.clearSelection()
                    //重新查找
                    queryRecord()
                })
            })
            .catch(() => {})
    }

    return {
        queryRecord,
        addRecord,
        editRecord,
        handleSubmit,
        delFlag,
        deleteRecord,
        deleteMulRecord
    }
}

/** DeBug */
export function CrudDebug(...params: any) {
    if (Params.value.debug) {
        console.info('%c【DeBug】', 'color:red', ...params)
    }
}
/** 初始化搜索、编辑参数 */
const initElementArrs = (source: SourceOptType) => {
    const { filter, editor, column, columns } = source
    // 初始化
    CrudState.value.ElementArrs = { filter: {}, editor: {}, column: {}, columns: {} }

    // 初始化搜索框数据（如果options设置了默认值，会自动携带上，会一直在。）
    let filterModel: any = {}
    filter.forEach((item) => {
        try {
            filterModel[item] = columns[item].searchDefaultValue
        } catch (error) {
            return console.error('columns中不存在参数：', item)
        }
        CrudState.value.ElementArrs.filter[item] = columns[item]
    })
    CrudState.value.initSearchModel = DeepClone(filterModel)
    //把初始化的值进行赋值
    CrudState.value.searchModel = DeepClone(CrudState.value.initSearchModel)

    // 初始化编辑框数据（新增时会自动输入的默认值）
    let editorModel: any = {}
    editor.forEach((item) => {
        try {
            editorModel[item] = columns[item].addDefaultValue
        } catch (error) {
            return console.error('columns中不存在参数：', item)
        }
        CrudState.value.ElementArrs.editor[item] = columns[item]
    })
    CrudState.value.initEditModel = DeepClone(editorModel)

    // 表格数据行
    column.forEach((item) => {
        CrudState.value.ElementArrs.column[item] = columns[item]
    })
    CrudState.value.ElementArrs.columns = columns

    // TODO:展示的表格列
    // CrudState.value.tableColumn = columns
}
/** 判断编辑中 是否有上传功能 */
const IsUploadEditor = computed(() => {
    let Keys: string[] = []
    // 如果编辑有图片预览，就自动绑定
    for (const key in CrudState.value.ElementArrs.editor) {
        if (Object.prototype.hasOwnProperty.call(CrudState.value.ElementArrs.editor, key)) {
            const item = CrudState.value.ElementArrs.editor[key]
            if (item.type === 'el-upload') {
                // 只要是上传，无论是生命卡片样式，都需要清空
                Keys.push(key)
            }
        }
    }
    return Keys
})
const CRUD_Notification = (title: string, type: 'success' | 'warning' | 'info' | 'error' = 'success') => {
    ElNotification({
        title,
        type,
        offset: 70,
        duration: 2500
    })
}

==> crud-el-0.2.30.tgz: package/src/CrudEl/Component/UseCrud.ts <==
import { computed, reactive, ref, toRefs, ToRefs } from 'vue'
import { ElMessage, ElMessageBox, ElNotification } from 'element-plus'
// 只做初始化引用
import { DefParams, DefCrudState, DefCrudProps } from './DefConfig'
import { DataMerger, DeepClone, isObjEqual } from './Methods'
import { useHooks, hooksName } from './Hooks'

/** 经过main.ts初始化后去除响应式的DefParams值 */
export let defParams: SystemParametersType

/** main初始化时传入的函数 */
export const CrudRequest = ref()
/** 系统参数 */
const Params = ref<SystemParametersType>({ ...DefParams })
/** 组件参数 */
const CrudState = ref<CrusStateType>({ ...DefCrudState })
/** 组件Props */
const CrudProps = ref<CrudPropsType>({ ...DefCrudProps })

/** 系统培训初始化函数 */
export const InitSys = (Options: any) => {
    CrudRequest.value = Options.CrudRequest
    Params.value = DataMerger(DefParams, Options)
    defParams = { ...Params.value }
}
/** Props传参初始化 */
export const Initializer = (props: any) => {
    CrudProps.value = { ...props }
    CrudState.value = DeepClone(DefCrudState)
    // 初始化查询、编辑、表格列
    initElementArrs(CrudProps.value!.source)
    // 重置系统参数
    Params.value = DataMerger(Params.value, props)
}
// 必须使用toRefs，不然参数就没有响应式了，
/** 系统参数 */
export function UseParam() {
    return { ...toRefs(Params.value) }
}
/** 组件参数 */
export function UseState(): ToRefs<CrusStateType> {
    return { ...toRefs(CrudState.value) }
}
/** Props参数 */
export function UseProps() {
    return { ...toRefs(CrudProps.value) }
}

/** 增删改查等操作方法 */
export const UseFun = () => {
    /**
     * 在JSON中提取出主键对象
     * @param data 提交时的JSON对象，包含主键参数、值
     * @returns 返回包含主键的对象
     */
    const KeyHandling = (data: any) => {
        /** 返回包含主键的对象 */
        let keys: any = {}
        if (CrudProps.value.rowKey) keys[CrudProps.value.rowKey] = data[CrudProps.value.rowKey]
        if (CrudProps.value.rowKeys) {
            CrudProps.value.rowKeys.forEach((key: any) => {
                keys[key] = data[key]
            })
        }
        return keys
    }
    const { hookHandler } = useHooks(CrudProps.value.hooks)

    /** 查询函数 */
    function queryRecord() {
        CrudState.value.tableLoading = true

        /** 查询请求携带的数据 */
        const Data = reactive({ ...CrudState.value.searchModel, ...CrudState.value.searchFormValue })

        for (const key in Data) {
            // 处理下拉选择器清空后，携带空字符串发起请求的问题
            if (typeof Data[key] === 'string') {
                Data[key] = Data[key].trim()
                if (Data[key] === '') {
                    Data[key] = undefined
                }
            }
            if (Data[key] === null) {
                Data[key] = undefined
            }
        }

        // 查询前钩子
        if (!hookHandler(hooksName.beforeQueryHandler, { CrudState: CrudState, Data })) {
            return
        }

        // 开始查询
        CrudRequest.value({
            url: CrudProps.value.url,
            name: Params.value.ApiName.query,
            data: {
                pagenum: Params.value.Page_PageParams.current, //当前页
                pagesize: Params.value.Page_PageParams.pageSize, //分页大小
                ...Data // 查询参数
            }
        })
            .then((Res: { data?: any; page?: any }) => {
                /** 解构出后端返回的 data数组 与 page分页参数 */
                const { data, page } = Res
                // 处理响应的数据（表格赋值等）
                setTimeout(() => {
                    // 为了保证钩子的数据格式统计，加了一层
                    responseData(data, Data)
                    // 更新分页数据
                    Params.value.Page_PageParams.total = page.total
                    // 关闭表格加载
                    CrudState.value.tableLoading = false
                }, 50)
            })
            .catch(() => {
                CrudState.value.tableLoading = false
            })
    }
    /** 查询后 响应数据处理 */
    const responseData = (Data: any, reqDataSource: any) => {
        // 如果请求的是原始数据，就存下来（原始数据判断依据：请求没有添加任何参数）
        let originalArray = Object.keys(reqDataSource).filter((r) => reqDataSource[r] != undefined)
        if (originalArray.length == 0) {
            CrudState.value.originalData = Data
        }
        CrudState.value.originalData = DeepClone(Data)

        // 查询后钩子
        if (!hookHandler(hooksName.afterQueryHandler, { CrudState: CrudState, Data })) {
            return
        }

        // 表格赋值
        CrudState.value.tableData = DeepClone(Data)
    }
    /**点击新增按钮，打开编辑对话框，并初始化值 */
    function addRecord() {
        // 初始化新增时表单的数据
        CrudState.value.editModel = DeepClone(CrudState.value.initEditModel)
        const Data = reactive({ ...CrudState.value.editModel })

        if (!hookHandler(hooksName.beforeAddHandler, { CrudState: CrudState, Data })) {
            return
        }
        CrudState.value.Mode = 'add'
        CrudState.value.DialogVisible = true
    }
    /**点击编辑按钮，打开编辑对话框【把请求到的值赋值给 editModel】 */
    const editRecord = (row: any) => {
        // 获取到当前行的数据
        const Data = reactive({
            ...row
        })

        if (!hookHandler(hooksName.beforeEditHandler, { CrudState: CrudState, Data })) {
            return
        }

        const DeepData = DeepClone(Data)
        // 赋值给编辑表单
        Object.keys(DeepData).forEach((key) => {
            CrudState.value.editModel[key] = DeepData[key]
            CrudState.value.editFormValue[key] = DeepData[key]
        })

        CrudState.value.Mode = 'edit'
        CrudState.value.DialogVisible = true
    }
    /**处理表单提交动作 */
    const handleSubmit = async (formEl: any) => {
        if (!formEl) return
        await formEl.validate((valid: any) => {
            if (valid) {
                const Data = reactive({
                    ...CrudState.value.editModel
                })

                if (!hookHandler(hooksName.beforeSubmitHandler, { CrudState: CrudState, Data })) {
                    return
                }

                // 判断表单提交操作是新增还是编辑
                if (CrudState.value.Mode === 'add') {
                    handleAdd(Data)
                } else {
                    handleEdit(Data)
                }
            }
        })
    }
    /**处理表单新增动作 */
    const handleAdd = (value: any) => {
        CrudState.value.submitLoading = true

        let data = value

        // 判断编辑是否有 FormData 对象
        if (value.FormData instanceof FormData) {
            data = value.FormData
        }

        CrudRequest.value({ url: CrudProps.value.url, name: Params.value.ApiName.add, data })
            .then(() => {
                CrudState.value.DialogVisible = false
                CRUD_Notification('新增成功')
                queryRecord()
                CrudState.value.submitLoading = false
            })
            .catch(() => {
                // 请求数据时，后端返回错误代码
                CrudState.value.submitLoading = false
            })
    }
    /**处理表单编辑动作 */
    const handleEdit = (value: any) => {
        CrudState.value.submitLoading = false

        /** 最终提交时的参数 */
        let data = value

        // 判断编辑是否有 FormData 对象
        if (value.FormData instanceof FormData) {
            data = value.FormData
        } else {
            let Equal = data
            // 判断全量提交还是只提交修改项
            if (CrudProps.value.isObjEqual) {
            } else {
                // 主要解决了编辑表单时，只提交修改项
                Equal = isObjEqual(CrudState.value.editFormValue, data)
            }
            // 没有编辑项时，就不发送请求
            if (JSON.stringify(Equal) === '{}') {
                CrudState.value.submitLoading = false
                return ElMessage({ message: '没有提交项！', grouping: true })
            }

            data = {
                ...KeyHandling(value),
                ...Equal
            }
        }

        CrudRequest.value({ url: CrudProps.value.url, name: Params.value.ApiName.edit, data })
            .then(() => {
                CrudState.value.DialogVisible = false
                CRUD_Notification('编辑成功')
                queryRecord()
                CrudState.value.submitLoading = false
            })
            .catch(() => {
                // 请求数据时，后端返回错误代码
                CrudState.value.submitLoading = false
            })
    }
    /**标记删除 */
    const delFlag = (row: any) => {
        ElMessageBox.confirm(`确定要 “${row.deleteFlag ? '启用' : '禁用'}” 该条数据吗？`, '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                const Data = reactive({
                    ...KeyHandling(row),
                    deleteFlag: row.deleteFlag == 0 ? 1 : 0
                })
                if (!hookHandler(hooksName.beforedisabledHandler, { CrudState: CrudState, Data })) {
                    return
                }
                CrudRequest.value({
                    url: CrudProps.value.url,
                    name: Params.value.ApiName.deleteFlag,
                    data: Data
                }).then(() => {
                    CRUD_Notification(`数据${row.deleteFlag ? '启用' : '禁用'}成功`)
                    queryRecord()
                })
            })
            .catch(() => {})
    }
    /**删除单行数据 */
    const deleteRecord = (row: any) => {
        ElMessageBox.confirm('确定要删除本条数据吗？', '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                // 获取到需要删除的当前行的数据
                const Data = reactive({
                    ...KeyHandling(row)
                })

                // 删除前钩子
                if (!hookHandler(hooksName.beforeDeteleHandler, { CrudState: CrudState, Data })) {
                    return
                }
                CrudRequest.value({
                    url: CrudProps.value.url,
                    name: Params.value.ApiName.delete,
                    data: Data
                }).then(() => {
                    CRUD_Notification('删除成功')
                    queryRecord()
                })
            })
            .catch(() => {})
    }
    /**删除多行数据 */
    const deleteMulRecord = () => {
        // 整理批量删除的 id
        const multipleSelection = CrudState.value.multipleSelection.map((item: any) => {
            return item[CrudProps.value.rowKey!]
        })

        ElMessageBox.confirm(`确定要删除这 ${multipleSelection.length} 条数据吗？`, '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                // 获取到需要删除的当前行的数据
                const Data = reactive({
                    [CrudProps.value.rowKey!]: multipleSelection,
                    selection: CrudState.value.multipleSelection
                })

                // 批量删除之前钩子
                if (!hookHandler(hooksName.beforeBatchDeteleHandler, { CrudState: CrudState, Data })) {
                    return
                }

                CrudRequest.value({
                    url: CrudProps.value.url,
                    name: Params.value.ApiName.remove,
                    data: Data
                }).then(() => {
                    CRUD_Notification('批量删除成功')
                    CrudState.value.multipleSelection = []
                    //把已经选择的数据清空
                    CrudState.value.CompRef.TableRef.clearSelection()
                    //重新查找
                    queryRecord()
                })
            })
            .catch(() => {})
    }

    return {
        queryRecord,
        addRecord,
        editRecord,
        handleSubmit,
        delFlag,
        deleteRecord,
        deleteMulRecord
    }
}

/** DeBug */
export function CrudDebug(...params: any) {
    if (UseParam().debug) {
        console.info('%c【DeBug】', 'color:red', ...params)
    }
}
/** 初始化搜索、编辑参数 */
const initElementArrs = (source: SourceOptType) => {
    const { filter, editor, column, expand, columns } = source
    // 初始化
    CrudState.value.ElementArrs = { filter: {}, editor: {}, column: {}, expand: {}, columns: {} }

    // 初始化搜索框数据（如果options设置了默认值，会自动携带上，会一直在。）
    let filterModel: any = {}
    filter.forEach((item) => {
        try {
            filterModel[item] = columns[item].searchDefaultValue
        } catch (error) {
            return console.error('columns中不存在参数：', item)
        }
        CrudState.value.ElementArrs.filter[item] = columns[item]
    })
    CrudState.value.initSearchModel = DeepClone(filterModel)
    //把初始化的值进行赋值
    CrudState.value.searchModel = DeepClone(CrudState.value.initSearchModel)

    // 初始化编辑框数据（新增时会自动输入的默认值）
    let editorModel: any = {}
    editor.forEach((item) => {
        try {
            editorModel[item] = columns[item].addDefaultValue
        } catch (error) {
            return console.error('columns中不存在参数：', item)
        }
        CrudState.value.ElementArrs.editor[item] = columns[item]
    })
    CrudState.value.initEditModel = DeepClone(editorModel)

    if (expand) {
        console.log('expand', expand)

        // 表格展开行
        expand.forEach((item) => {
            CrudState.value.ElementArrs.expand[item] = columns[item]
        })
    }

    // 表格数据行
    column.forEach((item) => {
        CrudState.value.ElementArrs.column[item] = columns[item]
    })
    CrudState.value.ElementArrs.columns = columns

    // TODO:展示的表格列
    // CrudState.value.tableColumn = columns
}
/** 判断编辑中 是否有上传功能 */
const IsUploadEditor = computed(() => {
    let Keys: string[] = []
    // 如果编辑有图片预览，就自动绑定
    for (const key in CrudState.value.ElementArrs.editor) {
        if (Object.prototype.hasOwnProperty.call(CrudState.value.ElementArrs.editor, key)) {
            const item = CrudState.value.ElementArrs.editor[key]
            if (item.type === 'el-upload') {
                // 只要是上传，无论是生命卡片样式，都需要清空
                Keys.push(key)
            }
        }
    }
    return Keys
})
const CRUD_Notification = (title: string, type: 'success' | 'warning' | 'info' | 'error' = 'success') => {
    ElNotification({
        title,
        type,
        offset: 70,
        duration: 2500
    })
}
